@@ -36,16 +36,19 @@ from tests.fixtures.example_project import (
     example_changelog_rst,
 )
 from tests.fixtures.repos import (
-    get_versions_for_trunk_only_repo_w_no_tags,
-    get_versions_for_trunk_only_repo_w_prerelease_tags,
-    get_versions_for_trunk_only_repo_w_tags,
-    repo_w_git_flow_and_release_channels_angular_commits,
-    repo_w_git_flow_and_release_channels_angular_commits_using_tag_format,
-    repo_w_git_flow_and_release_channels_emoji_commits,
-    repo_w_git_flow_and_release_channels_scipy_commits,
     repo_w_git_flow_angular_commits,
     repo_w_git_flow_emoji_commits,
     repo_w_git_flow_scipy_commits,
+    repo_w_git_flow_w_alpha_prereleases_n_angular_commits,
+    repo_w_git_flow_w_alpha_prereleases_n_emoji_commits,
+    repo_w_git_flow_w_alpha_prereleases_n_scipy_commits,
+    repo_w_git_flow_w_beta_alpha_rev_prereleases_n_angular_commits,
+    repo_w_git_flow_w_beta_alpha_rev_prereleases_n_emoji_commits,
+    repo_w_git_flow_w_beta_alpha_rev_prereleases_n_scipy_commits,
+    repo_w_git_flow_w_rc_n_alpha_prereleases_n_angular_commits,
+    repo_w_git_flow_w_rc_n_alpha_prereleases_n_angular_commits_using_tag_format,
+    repo_w_git_flow_w_rc_n_alpha_prereleases_n_emoji_commits,
+    repo_w_git_flow_w_rc_n_alpha_prereleases_n_scipy_commits,
     repo_w_github_flow_w_default_release_channel_angular_commits,
     repo_w_github_flow_w_default_release_channel_emoji_commits,
     repo_w_github_flow_w_default_release_channel_scipy_commits,
@@ -72,9 +75,9 @@ from tests.util import (
 
 if TYPE_CHECKING:
     from pathlib import Path
+    from typing import TypedDict
 
     from click.testing import CliRunner
-    from git import Repo
     from requests_mock import Mocker
 
     from tests.e2e.conftest import RetrieveRuntimeContextFn
@@ -84,43 +87,50 @@ if TYPE_CHECKING:
         UseReleaseNotesTemplateFn,
     )
     from tests.fixtures.git_repo import (
-        BuildRepoFn,
+        BuildRepoFromDefinitionFn,
+        BuiltRepoResult,
+        CommitConvention,
+        CommitDef,
         CommitNReturnChangelogEntryFn,
         GetCommitDefFn,
-        GetVersionStringsFn,
+        GetRepoDefinitionFn,
+        GetVersionsFromRepoBuildDefFn,
     )
 
+    class Commit2Section(TypedDict):
+        angular: Commit2SectionCommit
+        emoji: Commit2SectionCommit
+        scipy: Commit2SectionCommit
+
+    class Commit2SectionCommit(TypedDict):
+        commit: CommitDef
+        section: str
+
 
 @pytest.mark.parametrize("arg0", [None, "--post-to-release-tag"])
 @pytest.mark.parametrize(
-    "repo, get_version_strings_fn",
+    "repo_result",
     [
-        (
-            lazy_fixture(repo_fixture),
-            lazy_fixture(get_versions_fn),
-        )
-        for repo_fixture, get_versions_fn in (
+        lazy_fixture(repo_fixture)
+        for repo_fixture in (
             # Only need to test when it has tags or no tags
             # DO NOT need to consider all repo types as it doesn't change no-op behavior
-            (
-                repo_w_no_tags_angular_commits.__name__,
-                get_versions_for_trunk_only_repo_w_no_tags.__name__,
-            ),
-            (
-                repo_w_trunk_only_angular_commits.__name__,
-                get_versions_for_trunk_only_repo_w_tags.__name__,
-            ),
+            repo_w_no_tags_angular_commits.__name__,
+            repo_w_trunk_only_angular_commits.__name__,
         )
     ],
 )
 def test_changelog_noop_is_noop(
-    repo: Repo,
-    get_version_strings_fn: GetVersionStringsFn,
+    repo_result: BuiltRepoResult,
     arg0: str | None,
     cli_runner: CliRunner,
+    get_versions_from_repo_build_def: GetVersionsFromRepoBuildDefFn,
 ):
-    if (version_str := get_version_strings_fn()[-1]) == "Unreleased":
-        version_str = None
+    repo = repo_result["repo"]
+    repo_def = repo_result["definition"]
+    released_versions = get_versions_from_repo_build_def(repo_def)
+
+    version_str = released_versions[-1] if len(released_versions) > 0 else None
 
     repo.git.reset("--hard")
 
@@ -168,7 +178,7 @@ def test_changelog_noop_is_noop(
     ],
 )
 @pytest.mark.parametrize(
-    "repo",
+    "repo_result",
     [
         *[
             lazy_fixture(repo_fixture)
@@ -201,16 +211,22 @@ def test_changelog_noop_is_noop(
                 repo_w_git_flow_angular_commits.__name__,
                 repo_w_git_flow_emoji_commits.__name__,
                 repo_w_git_flow_scipy_commits.__name__,
-                repo_w_git_flow_and_release_channels_angular_commits.__name__,
-                repo_w_git_flow_and_release_channels_emoji_commits.__name__,
-                repo_w_git_flow_and_release_channels_scipy_commits.__name__,
-                repo_w_git_flow_and_release_channels_angular_commits_using_tag_format.__name__,
+                repo_w_git_flow_w_beta_alpha_rev_prereleases_n_angular_commits.__name__,
+                repo_w_git_flow_w_beta_alpha_rev_prereleases_n_emoji_commits.__name__,
+                repo_w_git_flow_w_beta_alpha_rev_prereleases_n_scipy_commits.__name__,
+                repo_w_git_flow_w_alpha_prereleases_n_angular_commits.__name__,
+                repo_w_git_flow_w_alpha_prereleases_n_emoji_commits.__name__,
+                repo_w_git_flow_w_alpha_prereleases_n_scipy_commits.__name__,
+                repo_w_git_flow_w_rc_n_alpha_prereleases_n_angular_commits.__name__,
+                repo_w_git_flow_w_rc_n_alpha_prereleases_n_emoji_commits.__name__,
+                repo_w_git_flow_w_rc_n_alpha_prereleases_n_scipy_commits.__name__,
+                repo_w_git_flow_w_rc_n_alpha_prereleases_n_angular_commits_using_tag_format.__name__,
             ]
         ],
     ],
 )
 def test_changelog_content_regenerated(
-    repo: Repo,
+    repo_result: BuiltRepoResult,
     cli_runner: CliRunner,
     update_pyproject_toml: UpdatePyprojectTomlFn,
     changelog_file: Path,
@@ -270,14 +286,15 @@ def test_changelog_content_regenerated(
 )
 @pytest.mark.usefixtures(change_to_ex_proj_dir.__name__)
 def test_changelog_content_regenerated_masked_initial_release(
-    build_trunk_only_repo_w_tags: BuildRepoFn,
+    build_repo_from_definition: BuildRepoFromDefinitionFn,
+    get_repo_definition_4_trunk_only_repo_w_tags: GetRepoDefinitionFn,
     example_project_dir: ExProjectDir,
     cli_runner: CliRunner,
     changelog_file: Path,
     insertion_flag: str,
 ):
-    build_trunk_only_repo_w_tags(
-        dest_dir=example_project_dir,
+    build_definition = get_repo_definition_4_trunk_only_repo_w_tags(
+        commit_type="angular",
         mask_initial_release=True,
         extra_configs={
             "tool.semantic_release.changelog.default_templates.changelog_file": str(
@@ -286,6 +303,7 @@ def test_changelog_content_regenerated_masked_initial_release(
             "tool.semantic_release.changelog.mode": ChangelogMode.INIT.value,
         },
     )
+    build_repo_from_definition(example_project_dir, build_definition)
 
     # Because we are in init mode, the insertion flag is not present in the changelog
     # we must take it out manually because our repo generation fixture includes it automatically
@@ -323,7 +341,7 @@ def test_changelog_content_regenerated_masked_initial_release(
     ],
 )
 @pytest.mark.parametrize(
-    "repo",
+    "repo_result",
     [
         lazy_fixture(repo_fixture)
         for repo_fixture in [
@@ -334,7 +352,7 @@ def test_changelog_content_regenerated_masked_initial_release(
     ],
 )
 def test_changelog_update_mode_unchanged(
-    repo: Repo,
+    repo_result: BuiltRepoResult,
     cli_runner: CliRunner,
     update_pyproject_toml: UpdatePyprojectTomlFn,
     changelog_file: Path,
@@ -380,7 +398,7 @@ def test_changelog_update_mode_unchanged(
     ],
 )
 @pytest.mark.parametrize(
-    "repo",
+    "repo_result",
     [
         lazy_fixture(repo_fixture)
         for repo_fixture in [
@@ -394,7 +412,7 @@ def test_changelog_update_mode_unchanged(
     ],
 )
 def test_changelog_update_mode_no_prev_changelog(
-    repo: Repo,
+    repo_result: BuiltRepoResult,
     cli_runner: CliRunner,
     update_pyproject_toml: UpdatePyprojectTomlFn,
     changelog_file: Path,
@@ -451,7 +469,7 @@ def test_changelog_update_mode_no_prev_changelog(
     ],
 )
 @pytest.mark.parametrize(
-    "repo",
+    "repo_result",
     [
         lazy_fixture(repo_fixture)
         for repo_fixture in [
@@ -462,7 +480,7 @@ def test_changelog_update_mode_no_prev_changelog(
     ],
 )
 def test_changelog_update_mode_no_flag(
-    repo: Repo,
+    repo_result: BuiltRepoResult,
     cli_runner: CliRunner,
     update_pyproject_toml: UpdatePyprojectTomlFn,
     changelog_file: Path,
@@ -524,7 +542,7 @@ def test_changelog_update_mode_no_flag(
     ],
 )
 @pytest.mark.parametrize(
-    "repo",
+    "repo_result",
     [
         lazy_fixture(repo_fixture)
         for repo_fixture in [
@@ -536,19 +554,22 @@ def test_changelog_update_mode_no_flag(
     ],
 )
 def test_changelog_update_mode_no_header(
-    repo: Repo,
+    repo_result: BuiltRepoResult,
     cli_runner: CliRunner,
     update_pyproject_toml: UpdatePyprojectTomlFn,
     changelog_format: ChangelogOutputFormat,
     changelog_file: Path,
     default_md_changelog_insertion_flag: str,
     default_rst_changelog_insertion_flag: str,
+    get_versions_from_repo_build_def: GetVersionsFromRepoBuildDefFn,
 ):
     """
     Given a changelog template with the insertion flag at the beginning of the file,
     When the changelog command is run in "update" mode,
     Then the changelog is rebuilt with the latest release prepended to the existing content.
     """
+    repo = repo_result["repo"]
+
     # Mappings of correct fixtures to use based on the changelog format
     insertion_flags = {
         ChangelogOutputFormat.MARKDOWN: (
@@ -587,7 +608,8 @@ def test_changelog_update_mode_no_header(
         expected_changelog_content = rfd.read()
 
     # Reset changelog file to last release
-    repo.git.checkout(repo.tags[-2].name, "--", str(changelog_file.name))
+    previous_tag = f'v{get_versions_from_repo_build_def(repo_result["definition"])[-2]}'
+    repo.git.checkout(previous_tag, "--", str(changelog_file.name))
 
     # Act
     cli_cmd = [MAIN_PROG_NAME, CHANGELOG_SUBCMD]
@@ -622,7 +644,7 @@ def test_changelog_update_mode_no_header(
     ],
 )
 @pytest.mark.parametrize(
-    "repo",
+    "repo_result",
     [
         lazy_fixture(repo_fixture)
         for repo_fixture in [
@@ -634,20 +656,25 @@ def test_changelog_update_mode_no_header(
     ],
 )
 def test_changelog_update_mode_no_footer(
-    repo: Repo,
+    repo_result: BuiltRepoResult,
     cli_runner: CliRunner,
     update_pyproject_toml: UpdatePyprojectTomlFn,
     changelog_format: ChangelogOutputFormat,
     changelog_file: Path,
     insertion_flag: str,
+    get_versions_from_repo_build_def: GetVersionsFromRepoBuildDefFn,
 ):
     """
     Given a changelog template with the insertion flag at the end of the file,
     When the changelog command is run in "update" mode,
     Then the changelog is rebuilt with only the latest release.
     """
+    repo_result["repo"]
+
     # Mappings of correct fixtures to use based on the changelog format
-    prev_version_tag = repo.tags[-2].name
+    prev_version_tag = (
+        f"v{get_versions_from_repo_build_def(repo_result['definition'])[-2]}"
+    )
     split_flags = {
         ChangelogOutputFormat.MARKDOWN: f"\n\n## {prev_version_tag}",
         ChangelogOutputFormat.RESTRUCTURED_TEXT: f"\n\n.. _changelog-{prev_version_tag}:",
@@ -721,7 +748,7 @@ def test_changelog_update_mode_no_footer(
     ],
 )
 @pytest.mark.parametrize(
-    "repo",
+    "repo_result",
     [
         lazy_fixture(repo_fixture)
         for repo_fixture in [
@@ -733,7 +760,7 @@ def test_changelog_update_mode_no_footer(
     ],
 )
 def test_changelog_update_mode_no_releases(
-    repo: Repo,
+    repo_result: BuiltRepoResult,
     cli_runner: CliRunner,
     update_pyproject_toml: UpdatePyprojectTomlFn,
     changelog_file: Path,
@@ -821,7 +848,7 @@ def test_changelog_update_mode_no_releases(
     ],
 )
 @pytest.mark.parametrize(
-    "repo, commit_type",
+    "repo_result, commit_type",
     [
         (lazy_fixture(repo_fixture), repo_fixture.split("_")[-2])
         for repo_fixture in [
@@ -832,8 +859,8 @@ def test_changelog_update_mode_no_releases(
     ],
 )
 def test_changelog_update_mode_unreleased_n_released(
-    repo: Repo,
-    commit_type: str,
+    repo_result: BuiltRepoResult,
+    commit_type: CommitConvention,
     changelog_format: ChangelogOutputFormat,
     cli_runner: CliRunner,
     update_pyproject_toml: UpdatePyprojectTomlFn,
@@ -851,6 +878,8 @@ def test_changelog_update_mode_unreleased_n_released(
     When the changelog command is run in "update" mode,
     Then the changelog is only updated with the unreleased changes.
     """
+    repo = repo_result["repo"]
+
     # Set the project configurations
     update_pyproject_toml(
         "tool.semantic_release.changelog.mode", ChangelogMode.UPDATE.value
@@ -860,7 +889,7 @@ def test_changelog_update_mode_unreleased_n_released(
         str(changelog_file.name),
     )
 
-    commit_n_section = {
+    commit_n_section: Commit2Section = {
         "angular": {
             "commit": get_commit_def_of_angular_commit(
                 "perf: improve the performance of the application"
@@ -998,7 +1027,14 @@ def test_changelog_release_tag_not_in_history(
 
 
 @pytest.mark.usefixtures(repo_w_trunk_only_n_prereleases_angular_commits.__name__)
-@pytest.mark.parametrize("args", [("--post-to-release-tag", "v0.1.0")])
+@pytest.mark.parametrize(
+    "args",
+    [
+        ("--post-to-release-tag", "v0.1.0"),  #      first release
+        ("--post-to-release-tag", "v0.1.1-rc.1"),  # second release
+        ("--post-to-release-tag", "v0.2.0"),  #      latest release
+    ],
+)
 def test_changelog_post_to_release(args: list[str], cli_runner: CliRunner):
     # Set up a requests HTTP session so we can catch the HTTP calls and ensure they're
     # made
@@ -1082,17 +1118,12 @@ def test_changelog_post_to_release(args: list[str], cli_runner: CliRunner):
 
 
 @pytest.mark.parametrize(
-    "repo, get_version_strings",
-    [
-        (
-            lazy_fixture(repo_w_trunk_only_n_prereleases_angular_commits.__name__),
-            lazy_fixture(get_versions_for_trunk_only_repo_w_prerelease_tags.__name__),
-        ),
-    ],
+    "repo_result",
+    [lazy_fixture(repo_w_trunk_only_n_prereleases_angular_commits.__name__)],
 )
 def test_custom_release_notes_template(
-    repo: Repo,
-    get_version_strings: GetVersionStringsFn,
+    repo_result: BuiltRepoResult,
+    get_versions_from_repo_build_def: GetVersionsFromRepoBuildDefFn,
     use_release_notes_template: UseReleaseNotesTemplateFn,
     retrieve_runtime_context: RetrieveRuntimeContextFn,
     post_mocker: Mocker,
@@ -1100,11 +1131,13 @@ def test_custom_release_notes_template(
 ) -> None:
     """Verify the template `.release_notes.md.j2` from `template_dir` is used."""
     expected_call_count = 1
-    version = Version.parse(get_version_strings()[-1])
+    version = Version.parse(
+        get_versions_from_repo_build_def(repo_result["definition"])[-1]
+    )
 
     # Setup
     use_release_notes_template()
-    runtime_context = retrieve_runtime_context(repo)
+    runtime_context = retrieve_runtime_context(repo_result["repo"])
     release_history = get_release_history_from_context(runtime_context)
     release = release_history.released[version]
     tag = runtime_context.version_translator.str_to_tag(str(version))
